/*
     This file is part of GNUnet.
     (C) 2001, 2002 Christian Grothoff (and other contributing authors)

     GNUnet is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published
     by the Free Software Foundation; either version 2, or (at your
     option) any later version.

     GNUnet is distributed in the hope that it will be useful, but
     WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
     General Public License for more details.

     You should have received a copy of the GNU General Public License
     along with GNUnet; see the file COPYING.  If not, write to the
     Free Software Foundation, Inc., 59 Temple Place - Suite 330,
     Boston, MA 02111-1307, USA.
*/

/**
 * AFS CORE. This is the code that is plugged
 * into the GNUnet core to enable AFS.
 *
 * @author Christian Grothoff
 * @file applications/afs/protocol/afs.c
 **/

#include "config.h"

#include "util/configuration.h"
#include "util/logging.h"
#include "util/statuscalls.h"
#include "util/storage.h"
#include "util/returnvalues.h"
#include "util/xmalloc.h"
#include "applications/afs/util/policy.h"
#include "applications/afs/protocol/afs.h"
#include "applications/afs/protocol/routing.h"
#include "applications/afs/protocol/handler.h"
#include "applications/afs/protocol/querymanager.h"
#include "applications/afs/database/manager.h"
#include "applications/afs/database/fileindex.h"


/**
 * Global core API.
 **/
CoreAPIForApplication * coreAPI = NULL;

/**
 * Filters.
 **/
Bloomfilter * superBloomFilter;
Bloomfilter * singleBloomFilter;

/**
 * Initialize the AFS module. This method name must match
 * the library name (libgnunet_XXX => initialize_XXX).
 * @return SYSERR on errors
 **/
int initialize_afs_protocol(CoreAPIForApplication * capi) {
  int ok = OK;
  char * fn;

  fn = getFileName("AFS",
                   "AFSDIR",
                   "Configuration must specify directory for "\
                   "AFS data in section %s under %s.\n");
  mkdirp(fn);
  FREE(fn);
  coreAPI = capi;
  /* assuming that the CACHE_HASH_TABLE_SIZE is
     used to 75% for local content and to 25%
     for migrated content and keywords... */
  fn = getFileName("AFS",
		   "BLOOMFILTER1",
		   "You must specify a filename for the bloomfilter in section %s under %s");		     
  superBloomFilter 
    = loadBloomfilter(fn,
		SUPERBLOOMFILTER_SIZE,
		SUPERBLOOMFILTER_K);
  registerSpaceMonitor(fn);
  FREE(fn);
  fn = getFileName("AFS",
		   "BLOOMFILTER2",
		   "You must specify a filename for the bloomfilter in section %s under %s");
  singleBloomFilter 
    = loadBloomfilter(fn,
  		SINGLEBLOOMFILTER_SIZE,
		SINGLEBLOOMFILTER_K);
  registerSpaceMonitor(fn);
  FREE(fn);

  initFileIndex();
  initLookup();
  initAnonymityPolicy(coreAPI);
  initManager();
  initQueryManager();
  initRouting();
  initAFSHandler();
  LOG(LOG_DEBUG,
      "DEBUG: AFS registering handlers %d %d %d and %d %d %d %d %d %d\n",
      AFS_p2p_PROTO_QUERY,
      AFS_p2p_PROTO_3HASH_RESULT,
      AFS_p2p_PROTO_CHK_RESULT,
      AFS_CS_PROTO_QUERY,
      AFS_CS_PROTO_INSERT_CHK,
      AFS_CS_PROTO_INSERT_3HASH,
      AFS_CS_PROTO_INDEX_BLOCK,
      AFS_CS_PROTO_INDEX_FILE,
      AFS_CS_PROTO_INDEX_SUPER);
  /* p2p handlers */
  if (SYSERR == capi->registerHandler(AFS_p2p_PROTO_QUERY,
				      &handleQUERY))
    ok = SYSERR; 
  if (SYSERR == capi->registerHandler(AFS_p2p_PROTO_3HASH_RESULT,
				      &handle3HASH_CONTENT))
    ok = SYSERR; 
  if (SYSERR == capi->registerHandler(AFS_p2p_PROTO_CHK_RESULT,
				      &handleCHK_CONTENT))
    ok = SYSERR; 

  /* CS handlers */
  if (SYSERR == capi->registerClientHandler(AFS_CS_PROTO_QUERY,
					    (CSHandler)&csHandleRequestQuery))
    ok = SYSERR; 
  if (SYSERR == capi->registerClientHandler(AFS_CS_PROTO_INSERT_CHK,
					    (CSHandler)&csHandleRequestInsertCHK))
    ok = SYSERR; 
  if (SYSERR == capi->registerClientHandler(AFS_CS_PROTO_INSERT_3HASH,
					    (CSHandler)&csHandleRequestInsert3HASH))
    ok = SYSERR; 
  if (SYSERR == capi->registerClientHandler(AFS_CS_PROTO_INDEX_BLOCK,
					    (CSHandler)&csHandleRequestIndexBlock))
    ok = SYSERR;       
  if (SYSERR == capi->registerClientHandler(AFS_CS_PROTO_INDEX_FILE,
					    (CSHandler)&csHandleRequestIndexFile))
    ok = SYSERR; 
  if (SYSERR == capi->registerClientHandler(AFS_CS_PROTO_INDEX_SUPER,
					    (CSHandler)&csHandleRequestIndexSuper))
    ok = SYSERR; 
   return ok;
}

void done_afs_protocol() {
  freeBloomfilter(singleBloomFilter);
  freeBloomfilter(superBloomFilter);
  /* p2p handlers */
  coreAPI->unregisterHandler(AFS_p2p_PROTO_QUERY,
			     &handleQUERY);
  coreAPI->unregisterHandler(AFS_p2p_PROTO_3HASH_RESULT,
			     &handle3HASH_CONTENT);
  coreAPI->unregisterHandler(AFS_p2p_PROTO_CHK_RESULT,
			     &handleCHK_CONTENT);

  /* CS handlers */
  coreAPI->unregisterClientHandler(AFS_CS_PROTO_QUERY,
				   (CSHandler)&csHandleRequestQuery);
  coreAPI->unregisterClientHandler(AFS_CS_PROTO_INSERT_CHK,
				   (CSHandler)&csHandleRequestInsertCHK);
  coreAPI->unregisterClientHandler(AFS_CS_PROTO_INSERT_3HASH,
				   (CSHandler)&csHandleRequestInsert3HASH);
  coreAPI->unregisterClientHandler(AFS_CS_PROTO_INDEX_BLOCK,
				   (CSHandler)&csHandleRequestIndexBlock);
  coreAPI->unregisterClientHandler(AFS_CS_PROTO_INDEX_FILE,
				   (CSHandler)&csHandleRequestIndexFile);
  coreAPI->unregisterClientHandler(AFS_CS_PROTO_INDEX_SUPER,
				   (CSHandler)&csHandleRequestIndexSuper);
  doneQueryManager();
  doneRouting();
  doneManager();
  doneLookup();
  doneFileIndex();
  doneAnonymityPolicy();
  coreAPI = NULL;
}

/* end of afs.c */
