/*
     This file is part of GNUnet.
     (C) 2001, 2002 Christian Grothoff (and other contributing authors)

     GNUnet is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published
     by the Free Software Foundation; either version 2, or (at your
     option) any later version.

     GNUnet is distributed in the hope that it will be useful, but
     WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
     General Public License for more details.

     You should have received a copy of the GNU General Public License
     along with GNUnet; see the file COPYING.  If not, write to the
     Free Software Foundation, Inc., 59 Temple Place - Suite 330,
     Boston, MA 02111-1307, USA.
*/

/**
 * Determine the IP of the local machine. We have many 
 * ways to get that IP:
 * a) from the interface (ifconfig)
 * b) via DNS from our HOSTNAME (environment)
 * c) from the configuration (HOSTNAME specification or static IP)
 *
 * Which way applies depends on the OS, the configuration
 * (dynDNS? static IP? NAT?) and at the end what the user 
 * needs.
 *
 * @file util/identity.c
 * @author Christian Grothoff
 * @author Tzvetan Horozov
 **/

#include "config.h"

#include <sys/ioctl.h>
#include <unistd.h>
#include <netdb.h>
#include <net/if.h>
#include <stdlib.h>
#include <string.h>
#ifdef SOLARIS
#include <sys/sockio.h>
#endif

#include "util/ipcheck.h"
#include "util/logging.h"
#include "util/cron.h"
#include "util/configuration.h"
#include "util/returnvalues.h"
#include "util/xmalloc.h"

/**
 * Our current IP address.
 **/
static IPaddr myAddress;

/**
 * Obtain the identity information for the current node
 * (connection information), conInfo.
 * @return SYSERR on failure, OK on success
 **/
static int getAddressFromHostname(IPaddr * identity) {
  char * hostname;
  struct hostent * ip;

  hostname = getenv("HOSTNAME");  
  if (hostname == NULL) {
    LOG(LOG_ERROR,
	"ERROR: Environment variable HOSTNAME not set!\n");
    return SYSERR;
  }
  /* LOG(LOG_DEBUG,
      "DEBUG: looking up $HOSTNAME (%s) to obtain local IP\n",
      hostname); */
  ip = gethostbyname(hostname);
  if (ip == NULL) {
    LOG(LOG_ERROR,
	"ERROR: Couldn't find IP for host '%s' (h_errno=%d)",
	hostname, 
	h_errno);
    return SYSERR;
  }
  if (ip->h_addrtype != AF_INET) {
    LOG(LOG_ERROR,
	"ERROR: getAddress: h_addrtype is not AF_INET (%d)!?",
	ip->h_addrtype);
    return SYSERR;
  }
  memcpy(identity,
	 &((struct in_addr*)ip->h_addr_list[0])->s_addr,
	 sizeof(struct in_addr));
  return OK;
}

#define MAX_INTERFACES 16
static int getAddressFromIOCTL(IPaddr * identity) {
  struct ifreq ifr[MAX_INTERFACES];
  struct ifconf ifc;
  int sockfd,ifCount;
  int i;
  char * interfaces;

  interfaces = getConfigurationString("NETWORK",
				      "INTERFACE");
  if (interfaces == NULL) {
    LOG(LOG_ERROR,
	"ERROR: No interface specified in section NETWORK under INTERFACE!\n");
    return SYSERR; /* that won't work! */
  }

  sockfd = socket(PF_INET, SOCK_DGRAM, 0);
  if (sockfd == -1) {
    FREE(interfaces);
    return SYSERR;
  }
  ifc.ifc_len = sizeof(ifr);
  ifc.ifc_buf = (char*)&ifr;
  
  if (ioctl(sockfd, SIOCGIFCONF, &ifc) == -1) {
    LOG(LOG_WARNING,
	"WARNING: Could not obtain IP with ioctl\n");
    close(sockfd);
    FREE(interfaces);
    return SYSERR;
  }
  ifCount = ifc.ifc_len / sizeof(struct ifreq);
  
  /* first, try to find exatly matching interface */
  for(i=0;i<ifCount;i++){
    if (ioctl(sockfd, SIOCGIFADDR, &ifr[i]) != 0)
       continue;
    if (ioctl(sockfd, SIOCGIFFLAGS, &ifr[i]) != 0)
       continue;
    if (!(ifr[i].ifr_flags & IFF_UP))
       continue; 
    if (strcmp((char*) interfaces,
	       (char*) ifr[i].ifr_name) != 0)
      continue;
    memcpy(identity,
	   &(((struct sockaddr_in *)&ifr[i].ifr_addr)->sin_addr),
	   sizeof(struct in_addr));
    close(sockfd);
    FREE(interfaces);
    return OK;
  }
  LOG(LOG_WARNING,
      "WARNING: Could not find interface %s in IOCTL, trying to find another one.\n",
      interfaces);
  /* if no such interface exists, take any interface but loopback */
  for(i=0;i<ifCount;i++){
    if (ioctl(sockfd, SIOCGIFADDR, &ifr[i]) != 0)
       continue;
    if (ioctl(sockfd, SIOCGIFFLAGS, &ifr[i]) != 0)
       continue;
    if (!(ifr[i].ifr_flags & IFF_UP))
       continue;
    if (strncmp("lo", 
		(char*) ifr[i].ifr_name, 2) == 0)
      continue;
    memcpy(identity,
	   &(((struct sockaddr_in *)&ifr[i].ifr_addr)->sin_addr),
	   sizeof(struct in_addr));
    close(sockfd);
    FREE(interfaces);
    return OK;
  }

  close(sockfd);
  LOG(LOG_WARNING,
      "WARNING: Could not obtain IP for interface %s using ioctl.\n",
      interfaces);
  FREE(interfaces);
  return SYSERR;
}

/**
 * Get the IP address for the local machine.
 * @return SYSERR on error, OK on success
 **/
static int getAddress(IPaddr  * address){
  char * ipString;
  int retval;
  struct hostent * ip; /* for the lookup of the IP in gnunet.conf */

  retval = SYSERR;
  ipString = getConfigurationString("NETWORK",
				    "IP");
  if (ipString == NULL) {
    if (OK == getAddressFromIOCTL(address))
      retval = OK;
    else 
      retval = getAddressFromHostname(address);
  } else {
    /* LOG(LOG_DEBUG,
	"DEBUG: obtaining local IP address from hostname %s\n",
	ipString); */
    ip = gethostbyname(ipString);
    if (ip == NULL) {
      LOG(LOG_ERROR,
	  "ERROR: Couldn't resolve '%s' (%s)",
	  ipString, hstrerror(h_errno));
      retval = SYSERR;
    } else {
      if (ip->h_addrtype != AF_INET) {
	LOG(LOG_ERROR,
	    "ERROR: getAddress: h_addrtype is not AF_INET (%d)!?",
	    ip->h_addrtype);
	retval = SYSERR;
      } else {
	memcpy (address,
		&((struct in_addr*) ip->h_addr_list[0])->s_addr,
		sizeof(struct in_addr));
	retval = OK;
      }
    }
    FREE(ipString);
  }
  return retval;
}

static void cronRefreshAddress(void * unused) {
  LOG(LOG_CRON,
      "CRON: enter cronRefreshAddress\n");
  if (SYSERR == getAddress(&myAddress))
    LOG(LOG_FAILURE,
	"FAILURE: Could not determine IP address of the local machine!\n");
  LOG(LOG_CRON,
      "CRON: exit cronRefreshAddress\n");
}

/** 
 * Initialize identity module. Requires configuration.
 **/
void initAddress() {
  if (SYSERR == getAddress(&myAddress)) 
    errexit("FATAL: Could not find IP for this host. Please provide the IP in the configuration file.\n");  
  addCronJob(&cronRefreshAddress, 
  	     2 * cronMINUTES,
	     2 * cronMINUTES,
	     NULL);
}

/** 
 * Shutdown identity module.
 **/
void doneAddress() {
  delCronJob(&cronRefreshAddress, 
  	     2 * cronMINUTES,
	     NULL);
}

/**
 * Get the IP address for the local machine.
 * @return SYSERR on error, OK on success
 **/
int getPublicIPAddress(IPaddr * address) {
  memcpy(address,
	 &myAddress,
	 sizeof(IPaddr));
  return OK;
}

/* end of identity.c */
